// ---------------------------------------------------------------------
/*
 * Copyright 2010 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */
// ---------------------------------------------------------------------
#ifndef	CHarkFileIO_NH_h_defined
#define	CHarkFileIO_NH_h_defined
// ---------------------------------------------------------------------
#include "../config.h"

#ifdef ENABLE_HARKIO_NH

#include <iostream>
#include <fstream>
#include <vector>
#include <boost/algorithm/string.hpp>

#ifdef ENABLE_HARKIO3
extern "C"{
	#include "libharkio3.h"
}
#endif	

#include "MICARY_struct.h"
#include "MICARY_GHDSS_struct.h"

#define	FORMAT_TYPE_TF	2
#define FORMAT_TYPE_W	3

#define __FILE_FORMAT_MICARY	(0)
#define __FILE_FORMAT_HARK	(1)

#define __FILE_TYPE_MIC		(0)
#define __FILE_TYPE_NOISE	(1)

class CHarkFileIO 
{
public:
	CHarkFileIO();
	~CHarkFileIO();

	// tft@C̃I[v(cFilenameɊJĂꍇ̓X[)
	bool OpenTFfile(const char* cFilename);

	// tft@C̍ăI[v
	bool ReopenTFfile(const char* cFilename);
	
	// tft@C̃N[Y
	bool CloseTFfile();

	//t@Cǂݍ
	bool ReadHeader(const char* cFilename,
					MICARY_TFDBFileHeader *psTFDBHead) ;

	//wb_Ƃ̑Ӄf[^̓Ǎ
	bool ReadInfo(const char* cFilename,
					MICARY_TFDBFileHeader *psTFDBHead,
					MICARY_LocationArray * plocaSrcPos , 
					MICARY_LocationArray * plocaMicPos ,
		                        MICARY_IntArray *piaFreqLine,
		                        bool bInfo = true  );

	//f[^̓Ǎ
	bool ReadData(const char*cFilename,
					MICARY_ComplexArray *micary_cmpaData)  ;

	// eLXgt@C(xml)
	bool LoadPositionData(const char* cFilename,
					MICARY_LocationArray * plocaPos) ;

	// eLXgt@C
	bool LoadPositionData(const char* cFilename,
					MICARY_LocationArray * plocaPos,int iFileType) ;
				
	// NeighborsXgvZ
	bool CalcNeighbors(const char* cFilename,
						float thresh);

	// ݂TFidw肵posindex擾
	int GetPosIndex(int id);
	
	// ݂TFWw肵čŋߐڈʒuid擾
	int GetNearestPosID(MICARY_Location locCur);
					
	// t@C쐬
	bool WriteData(const char* cFilename,
						MICARY_TFDBFileHeader *psTFDBHead,
						MICARY_ComplexArray micary_cmpaData,
						MICARY_IntArray	micary_iaFreqLine,
						MICARY_LocationArray micary_locaSource,
						MICARY_LocationArray micary_locaMic,
						const char* cDate = "",
						const char* cLocation = "",
						const char* cRobot = "",
						const char* cArrayType = "",
						const char * cCreator = "",
						const char* cMemo = "") ;
					
	// o[
	char*	m_pcTFfilename;	///< TFt@C
	harkio_TransferFunction	*m_ptf;	///< TFt@C|C^
	int	m_iFileType;	///< t@C^Cv(FORMAT_TYPE_TF or FORMAT_TYPE_W)
	
	// NX擾NeighborƎ擾
	// ) m_pPrinbrsgpԂTFt@CȂƁBposeszTFƋLĂ
	harkio_Neighbors *m_pPrinbrs;	///< NXōČvZNeighbors
	float	m_fThresh;	///< XbVh([Nbh[m])
	
private:
	
	float _EuclidDistance(harkio_Position *pos, MICARY_Location locCur);
};

#endif // ENABLE_HARKIO_NH
#endif // CHarkFileIO_NH_h_defined
